// DOMWriterImpl.java
// Justin Ludiwg 1999.03.01

package edu.wooster.lu99jp26.link;

import com.docuverse.dom.DOM;
import com.docuverse.dom.DOMWriter;
import java.io.IOException;
import java.io.OutputStream;
import java.io.OutputStreamWriter;
import java.io.Writer;
import java.net.URL;
import java.net.URLConnection;
import org.w3c.dom.*;

public class DOMWriterImpl implements DOMWriter
{
	
	
	public DOMWriterImpl ()
	{
	
	}
	
	public boolean write (DOM dom, Node node, Object output)
	{
		if (output instanceof URL)
			return write (dom, node, (URL) output);
		
		else if (output instanceof OutputStream)
			return write (dom, node, (OutputStream) output);
		
		else if (output instanceof Writer)
			return write (dom, node, (Writer) output);
		
		return false;
	}
	
	public boolean write (DOM dom, Node node, URL output)
	{
		OutputStream o;
		
		try
		{
			URLConnection conn = output.openConnection ();
			o = conn.getOutputStream ();
		}
		catch (IOException ioe)
		{
			ioe.printStackTrace ();
			return false;
		}
		
		return write (dom, node, o);
	}
	
	public boolean write (DOM dom, Node node, OutputStream output)
	{
		return write (dom, node, new OutputStreamWriter (output));
	}
	
	public boolean write (DOM dom, Node node, Writer output)
	{
		try
		{
			switch (node.getNodeType ())
			{
				case Node.ATTRIBUTE_NODE:
					start_attribute ((Attr) node, output);
					break;

				case Node.CDATA_SECTION_NODE:
					start_cdata ((CDATASection) node, output);
					break;

				case Node.COMMENT_NODE:
					start_comment ((Comment) node, output);
					break;

				case Node.DOCUMENT_FRAGMENT_NODE:
					start_document_fragment ((DocumentFragment) node, output);
					break;

				case Node.DOCUMENT_NODE:
					start_document ((Document) node, output);
					break;

				case Node.DOCUMENT_TYPE_NODE:
					start_document_type ((DocumentType) node, output);
					break;

				case Node.ELEMENT_NODE:
					start_element ((Element) node, output);
					break;

				case Node.ENTITY_NODE:
					start_entity ((Entity) node, output);
					break;

				case Node.ENTITY_REFERENCE_NODE:
					start_entity_reference ((EntityReference) node, output);
					break;

				case Node.NOTATION_NODE:
					start_notation ((Notation) node, output);
					break;

				case Node.PROCESSING_INSTRUCTION_NODE:
					start_processing ((ProcessingInstruction) node, output);
					break;

				case Node.TEXT_NODE:
					start_text ((Text) node, output);
					break;
			}
			
			NodeList list = node.getChildNodes ();
			int len = list.getLength ();
			
			for (int i = 0; i < len; i++)
				write (dom, list.item (i), output);
			
			switch (node.getNodeType ())
			{
				case Node.ATTRIBUTE_NODE:
					end_attribute ((Attr) node, output);
					break;

				case Node.CDATA_SECTION_NODE:
					end_cdata ((CDATASection) node, output);
					break;

				case Node.COMMENT_NODE:
					end_comment ((Comment) node, output);
					break;

				case Node.DOCUMENT_FRAGMENT_NODE:
					end_document_fragment ((DocumentFragment) node, output);
					break;

				case Node.DOCUMENT_NODE:
					end_document ((Document) node, output);
					break;

				case Node.DOCUMENT_TYPE_NODE:
					end_document_type ((DocumentType) node, output);
					break;

				case Node.ELEMENT_NODE:
					end_element ((Element) node, output);
					break;

				case Node.ENTITY_NODE:
					end_entity ((Entity) node, output);
					break;

				case Node.ENTITY_REFERENCE_NODE:
					end_entity_reference ((EntityReference) node, output);
					break;

				case Node.NOTATION_NODE:
					end_notation ((Notation) node, output);
					break;

				case Node.PROCESSING_INSTRUCTION_NODE:
					end_processing ((ProcessingInstruction) node, output);
					break;

				case Node.TEXT_NODE:
					end_text ((Text) node, output);
					break;
			}
		output.flush ();
		}
		catch (IOException ioe)
		{
			ioe.printStackTrace ();
			return false;
		}
		
		
		return true;
	}
	
	protected void start_attribute (Attr node, Writer o)
	throws IOException
	{
		
	}
	
	protected void end_attribute (Attr node, Writer o)
	throws IOException
	{
		
	}

	protected void start_cdata (CDATASection node, Writer o)
	throws IOException
	{
		o.write ("<![CDATA[" + node.getData () + "]]>");
	}

	protected void end_cdata (CDATASection node, Writer o)
	throws IOException
	{
		
	}

	protected void start_comment (Comment node, Writer o)
	throws IOException
	{
		o.write ("<!--" + node.getData () + "-->");
	}

	protected void end_comment (Comment node, Writer o)
	throws IOException
	{
		
	}

	protected void start_document_fragment (DocumentFragment node, Writer o)
	throws IOException
	{
		
	}

	protected void end_document_fragment (DocumentFragment node, Writer o)
	throws IOException
	{
		
	}

	protected void start_document (Document node, Writer o)
	throws IOException
	{
		
	}

	protected void end_document (Document node, Writer o)
	throws IOException
	{
		
	}

	protected void start_document_type (DocumentType node, Writer o)
	throws IOException
	{
		o.write ("<!DOCTYPE " + node.getName () + ">");
		// we need the public & system id or the dtd itself
		// perhaps we need to read in dtd directly from original file
		// or overload something in the sax archetecture
		
		// note that in my Viewer class I write this information directly
		// from a source document, along with the other header information
	}

	protected void end_document_type (DocumentType node, Writer o)
	throws IOException
	{
		
	}

	protected void start_element (Element node, Writer o)
	throws IOException
	{
		o.write ("<" + node.getTagName ());
		
		NamedNodeMap map = node.getAttributes ();
		int len = map.getLength ();
		
		for (int i = 0; i < len; i++)
		{
			Attr a = (Attr) map.item (i);
			
			// if in original document
			if (a.getSpecified ())
				o.write (" " + a.getName () + "=\"" + a.getValue () + "\"");
		}
		
		o.write (node.hasChildNodes () ? ">" : "/>");
	}

	protected void end_element (Element node, Writer o)
	throws IOException
	{
		if (node.hasChildNodes ())
			o.write ("</" + node.getTagName () + ">");
	}

	protected void start_entity (Entity node, Writer o)
	throws IOException
	{
		
	}

	protected void end_entity (Entity node, Writer o)
	throws IOException
	{
		
	}

	protected void start_entity_reference (EntityReference node, Writer o)
	throws IOException
	{
		
	}

	protected void end_entity_reference (EntityReference node, Writer o)
	throws IOException
	{
		
	}

	protected void start_notation (Notation node, Writer o)
	throws IOException
	{
		
	}

	protected void end_notation (Notation node, Writer o)
	throws IOException
	{
		
	}

	protected void start_processing (ProcessingInstruction node, Writer o)
	throws IOException
	{
		// o.write ("<?" + node.getTarget () + " " + node.getData () + "?>\n");
		
		// commented because Viewer class copies this information
		// from the source file, along with other header information
	}

	protected void end_processing (ProcessingInstruction node, Writer o)
	throws IOException
	{
		
	}

	protected void start_text (Text node, Writer o)
	throws IOException
	{
		o.write (node.getData ());
	}

	protected void end_text (Text node, Writer o)
	throws IOException
	{
		
	}
}