// SAXParser.java
// Justin Ludiwg 1999.03.01

package edu.wooster.lu99jp26.link;

import com.jclark.xml.parse.OpenEntity;
import com.jclark.xml.sax.Driver;
import edu.wooster.lu99jp26.util.Function;
import java.io.FileInputStream;
import java.io.InputStream;
import java.io.IOException;
import java.net.URL;
import java.util.Hashtable;

public class SAXParser extends Driver
{
	protected static Hashtable publicResources = defaultPublicResources ();
	protected static URL staticBaseURL = null;
	
	
	/**
	 * creates new Hashtable of the default PUBLIC resources
	 * @return Hashtable new Hashtable
	 */
	public static Hashtable defaultPublicResources ()
	{
		Hashtable h = new Hashtable (5);
		
		h.put ("-//lu99jp26.wooster.edu//DTD cat-index 1.0//EN", 
			   Function.userDirToURL ("data/cat-index.dtd"));
		h.put ("-//lu99jp26.wooster.edu//DTD cat-links 1.0//EN", 
			   Function.userDirToURL ("data/cat-links.dtd"));
		h.put ("-//lu99jp26.wooster.edu//DTD cat-descriptions 1.0//EN", 
			   Function.userDirToURL ("data/cat-descriptions.dtd"));
		h.put ("-//lu99jp26.wooster.edu//DTD cat-prices 1.0//EN", 
			   Function.userDirToURL ("data/cat-prices.dtd"));
		h.put ("-//lu99jp26.wooster.edu//DTD cat-inventory 1.0//EN", 
			   Function.userDirToURL ("data/cat-inventory.dtd"));
		 
		return h;
	}
	
	/**
	 * sets the base URL for resolving SYSTEM identifiers
	 * @param baseURL URL the new base URL (may be null)
	 */
	public static void setBaseURL (URL baseURL)
	{
		staticBaseURL = baseURL;
	}
	
	
	public SAXParser ()
	{
		super ();
	}
	
	/**
	 * Opens an external entity.
	 * @param systemId the system identifier specified in the entity declaration
	 * @param baseURL the base URL relative to which the system identifier
	 * should be resolved; null if no base URL is available
	 * @param publicId the public identifier specified in the entity declaration;
	 * null if no public identifier was specified
	 */
	public OpenEntity open (String systemId, URL baseURL, String publicId)
	throws IOException
	{
		URL url;
		InputStream in;
		
		try
		{
			url = resolve (publicId, systemId, baseURL);
			in = url.openStream ();
		}
		// MalformedURLException, IOException, NullPointerException
		catch (Exception e)
		{
			url = new URL (Function.userDirToURL ("data/default.xml"));
			in = url.openStream ();
			e.printStackTrace ();
		}
		
		return new OpenEntity (in, url.toString (), url);
	}
	
	/**
	 * takes a PUBLIC identifier and resolves it,
	 * or returns the SYSTEM identifier
	 * @param publicId String the PUBLIC identifier
	 * @param systemId String the SYSTEM identifier
	 * @param baseURL URL the current base URL
	 * @return String the URL of the PUBLIC resource,
	 * or the default SYSTEM resource
	 */
	public URL resolve (String publicId, String systemId, URL baseURL)
	throws IOException
	{
		if (publicId != null)
		{
			Object localId = publicResources.get (publicId);
			
			if (localId != null)
				systemId = (String) localId;
		}
		
		if (staticBaseURL != null)
			baseURL = staticBaseURL;
		
		return new URL (baseURL, systemId);
	}
}