// ViewerWindow.java
// Justin Ludiwg 1999.03.01

package edu.wooster.lu99jp26.link;

import edu.wooster.lu99jp26.util.Function;
import java.awt.Dimension;
import java.awt.Frame;
import java.awt.Menu;
import java.awt.MenuBar;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.WindowEvent;
import java.awt.event.WindowListener;
import javax.swing.JScrollPane;
import javax.swing.JViewport;

public class ViewerWindow extends Frame
implements ActionListener, WindowListener
{
	protected static int count = 0;
	
	protected int number;
	protected Viewer viewer;
	
	
	public ViewerWindow ()
	{
		this (new Viewer ());
	}
	
	public ViewerWindow (Viewer viewer)
	{
		super ();
		addWindowListener (this);
		number = ++count;
		
		// create menubar
		Menu file = new Menu ("File");
		Function.addMenuItem (file, "Open Local...", 'O', App.getApp ());
		Function.addMenuItem (file, "Retrieve From URL...", 'R', App.getApp ());
		Function.addMenuItem (file, "Close Window", 'W', this);
		file.addSeparator ();
		Function.addMenuItem (file, "Quit", 'Q', App.getApp ());
		
		MenuBar bar = new MenuBar ();
		bar.add (file);
		setMenuBar (bar);
		
		// create view pane & make it scrollable
		this.viewer = viewer;
		JScrollPane scroller = new JScrollPane ();
		JViewport port = scroller.getViewport ();
		port.add (viewer);
		
		// show window
		add (scroller);
		pack ();
		move (10 * count, 20 * count + 20);
		show ();
	}
	
	/**
	 * gets the viewer contained by this Window
	 * @return Viewer
	 */
	public Viewer getViewer ()
	{
		return viewer;
	}
	
	public Dimension preferredSize ()
	{
		return new Dimension (600, 400);
	}

    /**
     * Invoked when an action occurs.
     */
	public void actionPerformed (ActionEvent e)
	{
		String command = e.getActionCommand ();
		
		if (command.startsWith ("Close"))
			windowClosing (new WindowEvent (this, WindowEvent.WINDOW_CLOSING));
	}
	
	/**
     * Invoked when a window has been opened.
     */
    public void windowOpened (WindowEvent e) {}

    /**
     * Invoked when a window is in the process of being closed.
     * The close operation can be overridden at this point.
     */
    public void windowClosing (WindowEvent e)
    {
    	// dispose ();
    	setVisible (false);
    	// we're keeping xlinked documents open for entire session
    }

    /**
     * Invoked when a window has been closed.
     */
    public void windowClosed (WindowEvent e) {}

    /**
     * Invoked when a window is iconified.
     */
    public void windowIconified (WindowEvent e) {}

    /**
     * Invoked when a window is de-iconified.
     */
    public void windowDeiconified (WindowEvent e) {}

    /**
     * Invoked when a window is activated.
     */
    public void windowActivated (WindowEvent e) {}

    /**
     * Invoked when a window is de-activated.
     */
    public void windowDeactivated (WindowEvent e) {}
}